#!/bin/bash
##################################################################
# Licensed Materials - Property of IBM
# 5724-W78
# (C) Copyright IBM Corp. 2010
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
##################################################################

#
# Name: configureTSAM.sh
# 
# This file is directly called by the user or by the launchpad.
# If one of the steps failed and the user restarts the 
# script after repairing the error, he is asked if he wants to
# proceed on the step which failed, or if he wants to start
# from the beginning. The script also stores log files
# into the Tivoli Service Automation Manager logs file directory
# which is /opt/IBM/tsam/logs by default
# for each step performed. If something failed, the user
# sees the name of the log files linked to the failed step
# and also the last ten lines of these log files.
# All variables are read from the script setupEnv.sh. Check setupEnv.sh
# for a list of variables to be changed if you want to call this script
# without the launchpad.
# The i18n is done by the script i18n.sh.
# The following steps are performed, with their unique id:
# uniq id                               script called
# ------------------------------------  --------------------------------------
# CHECK_MIDDLEWARE                      checkMiddleware
# START_PREPARE_TIO                     startTIO
# CHANGE_APPLICATION_CLASSLOADER        changeApplClassLoaderOrderToPARENT
# CHANGE_APPLICATION_WAR_CLASSLOADER    changeApplWARClassLoaderPolicyToSINGLE
# ENABLE_CLASSLOADER_RELOAD             enableApplClassReloading
# STOP_RESTART_TIO                      stopTIO
# START_RESTART_TIO                     startTIO
# CREATE_CLOUD_MGMT                     createCloudManagementSubsystemDB
# ESTABLISH_TRUST_REL                   establishTrustRelationshipForTSAM
# CREATE_JDBCPROVIDER                   createJDBCProvider
# CREATE_DATASOURCE                     createDataSource
# CREATE_TIMERMANAGER                   createTimerManager
# CREATE_GLOBALVARIABLE_TIOHOME         createGlobalVariableTIO_HOME
# DISABLE_LDAP_CACHE                    disableLDAPCaches
# SYNC_NODE                             syncNode
# STOP_APPLICATION_CDS                  stopApplicationCDS
# DISABLE_CDS_AUTOSTART                 disableAutoStartForApplicationCDS
# SSHKEY_STOP_TIO                       stopTIO
# SSHKEY_CONFIG                         configureSSHkey
# SSHKEY_START_TIO                      startTIO
# INSERT_CREDENTIALS_TIO                insert_tioCredentials
# INSERT_CREDENTIALS_SSHSERVER_MASTER   update_sshServerMasterCredentials
# INSERT_CREDENTIALS_SSHCLIENT_MASTER   update_sshClientMasterCredentials
# INSERT_CREDENTIALS_SSHCLIENT_RSA      insert_sshClientRSACredentials
# INSERT_CREDENTIALS_SSHCLIENT          insert_sshClientCredentials
# INSERT_CREDENTIALS_SCPCLIENT_RSA      insert_scpClientRSACredentials
# INSERT_CREDENTIALS_SCPCLIENT          insert_scpClientCredentials
# STOP_APPLICATION_CLOUD                stopApplicationCloudEAR
# STOP_APPLICATION_MAXIMO               stopApplicationMAXIMO
# START_APPLICATION_MAXIMO              startApplicationMAXIMO
# START_APPLICATION_CLOUD               startApplicationCloudEAR
# START_PROPERTIES_TIO                  startTIO
# HYPERVISOR_MANAGER_PERMISSION_REMOVE  removeManageHyperVisorPermission
# GENERATE_IHSPLUGIN                    generateIHSPlugin


workDir=`dirname "$0"`
SCRIPT_NAME=$0

if [[ "$workDir" =~ ^/ ]]
then
	export TSAM_CONFIG_HOME=$workDir
else
	export TSAM_CONFIG_HOME="$(pwd)"/"${workDir}"
fi

. "$workDir/setupEnv.sh"

# Enable Localization
. "$workDir/../tools/i18n.sh" configureTSAM

echo

# The first parameter is the scriptname without the ending .sh
function callStep
{
	CURRENT_STEP_ID=$1
	CURRENT_STEP=$2
	CURRENT_SCRIPTNAME=${CURRENT_STEP}.sh
	LOGFILE_NAME_PREFIX=${CURRENT_STEP_ID}_${CURRENT_STEP}_${TIMESTAMP}
	if ! test -f "$TSAM_POSTINSTALL_PROGRESS" || ! cat "$TSAM_POSTINSTALL_PROGRESS" | egrep "^$CURRENT_STEP_ID$" 1>/dev/null 2>/dev/null
	then
		"$workDir/${CURRENT_SCRIPTNAME}" 1>>${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.log 2>>${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.err
		RC=$?
		if ! [[ "$RC" == "0" ]]
		then
			printLocalizedMessage errorMessage
			echo ${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.log
			echo ${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.err
			echo
			printLocalizedMessage logTitle
			echo ${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.log:
			tail -n 10 "${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.log"
			echo
			echo ${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.err:
			tail -n 10 "${TSAM_LOG}/${LOGFILE_NAME_PREFIX}.err"
			
			echo
			printLocalizedMessage fixAndRestart
			printLocalizedMessage pressAnyKey
			read

			exit $RC
		fi
		echo $CURRENT_STEP_ID >> "$TSAM_POSTINSTALL_PROGRESS"
	fi

}

function unittest
{
	if test -f "$workDir"/$(basename "$SCRIPT_NAME") && ! printLocalizedMessage a | grep "^[*][*][*]" 1>/dev/null 2>/dev/null
	then
		exit 0
	else
		exit 1
	fi
}

if [[ "$1" == "--test" ]]
then
	unittest
fi

printLocalizedMessage logDestination

mkdir -p $TSAM_LOG

if test -f "$TSAM_POSTINSTALL_PROGRESS"
then
	printLocalizedMessage warningNotClean
	read KEY
	if [[ "$KEY" == "y" ]]
	then
		printLocalizedMessage postInstallRestartLabel
		rm -f "$TSAM_POSTINSTALL_PROGRESS"
	fi	
fi

echo
printLocalizedMessage middlewareCheckLabel
callStep CHECK_MIDDLEWARE checkMiddleware

printLocalizedMessage stepTSAMcomponentsLabel

callStep START_PREPARE_TIO startTIO
callStep CHANGE_APPLICATION_CLASSLOADER changeApplClassLoaderOrderToPARENT
callStep CHANGE_APPLICATION_WAR_CLASSLOADER changeApplWARClassLoaderPolicyToSINGLE
callStep ENABLE_CLASSLOADER_RELOAD enableApplClassReloading
callStep STOP_RESTART_TIO stopTIO
callStep START_RESTART_TIO startTIO

printLocalizedMessage stepCreateDB

callStep CREATE_CLOUD_MGMT createCloudManagementSubsystemDB

printLocalizedMessage stepWASRelationship

callStep ESTABLISH_TRUST_REL establishTrustRelationshipForTSAM

printLocalizedMessage stepWASConfigTPM

callStep CREATE_JDBCPROVIDER createJDBCProvider
callStep CREATE_DATASOURCE createDataSource
callStep CREATE_TIMERMANAGER createTimerManager
callStep CREATE_GLOBALVARIABLE_TIOHOME createGlobalVariableTIO_HOME
callStep DISABLE_LDAP_CACHE disableLDAPCaches
callStep SYNC_NODE syncNode
callStep STOP_APPLICATION_CDS stopApplicationCDS
callStep DISABLE_CDS_AUTOSTART disableAutoStartForApplicationCDS

printLocalizedMessage stepSSHUpdate

callStep SSHKEY_STOP_TIO stopTIO
callStep SSHKEY_CONFIG configureSSHkey
callStep SSHKEY_START_TIO startTIO

printLocalizedMessage stepTPMCredentials

callStep INSERT_CREDENTIALS_TIO insert_tioCredentials
callStep INSERT_CREDENTIALS_SSHSERVER_MASTER update_sshServerMasterCredentials
callStep INSERT_CREDENTIALS_SSHCLIENT_MASTER update_sshClientMasterCredentials
callStep INSERT_CREDENTIALS_SSHCLIENT_RSA insert_sshClientRSACredentials
callStep INSERT_CREDENTIALS_SSHCLIENT insert_sshClientCredentials
callStep INSERT_CREDENTIALS_SCPCLIENT_RSA insert_scpClientRSACredentials
callStep INSERT_CREDENTIALS_SCPCLIENT insert_scpClientCredentials

printLocalizedMessage startDiscoveryWorkflow

callStep STOP_APPLICATION_CLOUD stopApplicationCloudEAR
callStep STOP_APPLICATION_MAXIMO stopApplicationMAXIMO
callStep START_APPLICATION_MAXIMO startApplicationMAXIMO
callStep START_APPLICATION_CLOUD startApplicationCloudEAR
callStep START_PROPERTIES_TIO startTIO
callStep START_DISCOVERY_WORKFLOW startTPMDiscoveryWorkflow

printLocalizedMessage stepRemoveHypervisorPermDB

callStep HYPERVISOR_MANAGER_PERMISSION_REMOVE removeManageHyperVisorPermission

printLocalizedMessage stepWASPlugin

callStep GENERATE_IHSPLUGIN generateIHSPlugin

printLocalizedMessage stepFilePermission

callStep SET_FILE_PERMISSIONS setFilePermissions

printLocalizedMessage pressAnyKey
read
