#!/bin/bash
##################################################################
# Licensed Materials - Property of IBM
# 5724-W78
# (C) Copyright IBM Corp. 2010
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
##################################################################

# Set variables to default values
RC_OK=0
RC_WARNING=10
RC_ERROR=20
RC_UNKNOWNOS=99

STATUS=$RC_OK

SILENT=0
SEPERATED=0
OUTPUTLIST=""
CHECK_OS=0
CHECK_MIDDLEWARE=0
DEBUG=0

TMPFILE=/tmp/tsam_lp.tmp

# Print text on screen if not called in silent mode
function printOnScreen
{
   if [ "$SILENT" == "0" ] && [ "$SEPERATED" == "0" ]
   then
      echo -en $1
   fi
	   
}

# If $DEBUG is 0, return false, if $DEBUG is 1, return true
function isDebug
{
        RETVAL=$(($(($DEBUG+1))%2))
        return $RETVAL
}

# Print usage on screen
function printUsage
{
   printOnScreen "Usage:\n"
   printOnScreen "   $0 [-c] [-s] -m\n"
   printOnScreen "   $0 [-c] [-s] -o\n"
}

while [ "$#" -ne "0" ]
do
   case $1 in
      "-s") SILENT=1; shift;;
      "-c") SEPERATED=1; shift;;
      "-m") CHECK_MIDDLEWARE=1; shift;;
      "-o") CHECK_OS=1; shift;;
      "-d") DEBUG=1; shift;;
      *) printUsage; exit 1;;
   esac
done

if [[ "$CHECK_MIDDLEWARE" == "0" ]] && [[ "$CHECK_OS" == "0" ]]
then
   printUsage;
fi


function isCheckMiddleware
{
   RETVAL=1;
   
   if [ "$CHECK_MIDDLEWARE" == "1" ]
   then
      RETVAL=0;
   fi
   
   return $RETVAL
}

function isCheckOperatingSystem
{
   RETVAL=1;
   
   if [ "$CHECK_OS" == "1" ]
   then
      RETVAL=0;
   fi
   
   return $RETVAL
}

function printOutputList
{
   if [ "$SILENT" == "0" ] && [ "$SEPERATED" == "1" ]
   then
      echo ${OUTPUTLIST} > ${TMPFILE}
   fi
}

function addToList
{
   if [ "$OUTPUTLIST" == "" ]
   then
      OUTPUTLIST=$1
   else
      OUTPUTLIST=${OUTPUTLIST},$1
   fi
}

function isAIX53
{
   retVal=1
   
   if [ "`uname -v``uname -r`" == "53" ]
   then
      retVal=0
   fi
   return $retVal   
}

function isAIX61
{
   retVal=1
   
   if [ "`uname -v``uname -r`" == "61" ]
   then
      retVal=0
   fi
   return $retVal   
}

function verify_os
{
   printOnScreen "Verifying operating System..."
   
   if isDebug || ! uname -s | grep -i AIX 1>/dev/null 2>/dev/null || ! prtconf -k | grep 64 1>/dev/null 2>/dev/null
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_UNKNOWNOS}
      addToList os
   else
      printOnScreen "OK\n"
   fi
}

function verify_disabledautomount
{
   printOnScreen "Verifying the AIX automount daemon..."
   
   if isDebug || lssrc -s automountd | grep active 1>/dev/null 2>/dev/null
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList automount
   else
      printOnScreen "OK\n"
   fi
}

function verify_pagingspace
{
   printOnScreen "Verifying the paging space..."
   
   VALUE=`lsps -s | sed "s/  */ /g" | cut -d ' ' -f 2 | tail -n1 | sed "s/^\([0-9]*\)\([A-Z]*\)/\1/"`
   TYPE=`lsps -s | sed "s/  */ /g" | cut -d ' ' -f 2 | tail -n1 | sed "s/^\([0-9]*\)\([A-Z]*\)/\2/"`
   
   MEMVALUE=$[`bootinfo -r` / 1024]
   
   if isDebug || ! [ "$TYPE" == MB ] || ! [ "$VALUE" -ge "$MEMVALUE" ]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList pagingspace
   else
      printOnScreen "OK\n"
   fi
}
   
function verify_maxprocperuser
{
   printOnScreen "Verifying the maximum number of process per user..."
   
   if isDebug || ! [ "`lsattr -E -l sys0 -a maxuproc -F value`" -ge "2048" ]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList maxprocperuser
   else
      printOnScreen "OK\n"
   fi
}

function verify_largefilesizes
{
   printOnScreen "Verifying large file sizes in the file system..."
   
   if ! isDebug && [ "`mount | sed "s/  */;/g" | grep "jfs;"`" == "" ]
   then
      printOnScreen "OK\n"
   else
      printOnScreen "\n"
      
      for FILESYSTEM in `mount | sed "s/  */;/g" | grep "jfs;" | cut -d ';' -f 2`
      do
         printOnScreen " Checking ${FILESYSTEM}..."
         if ! lsfs -c -q $FILESYSTEM | grep "bf true" 1>/dev/null 2>/dev/null
         then
            STATUS=${RC_ERROR}
            printOnScreen "ERROR\n"
            addToList "largefilesizes!${FILESYSTEM}"
         else
            printOnScreen "OK\n"
         fi
      done
   fi
   
}

function verify_userlimits
{
   printOnScreen "Verifying user limits for root..."
   verify_limits_for_user root: 1>/dev/null 2>/dev/null
   if ! [ "$?" == "0" ] || isDebug
   then
      STATUS=$RC_ERROR
      printOnScreen "ERROR\n"
      addToList "userlimits root"
   else
      printOnScreen "OK\n"
   fi
   
   
   printOnScreen "Verifying user limits for ctginst1..."
   verify_limits_for_user ctginst1: 1>/dev/null 2>/dev/null
   if ! [ "$?" == "0" ] || isDebug
   then
      STATUS=$RC_ERROR
      printOnScreen "ERROR\n"
      addToList "userlimits ctginst1" 
   else
      printOnScreen "OK\n"
   fi
   
}

function verify_limits_for_user
{

awk '
   BEGIN {
      INBLOCK = 0
      SEARCHPATTERN = "^.*"ARGV[2]
      FS=" *= *"
   }
   {
      if ( INBLOCK == 1 && $0 !~ /^[^:]+:/ && $0 ~ /.*=.*/ )
      {
         if ( $0 ~ /^.*nofiles *=/ )
         {
            if ( $2 < 8192 && $2 != -1 )
            {
               exit 1
            }
         }
         else
         {
            if ( $0 !~ /^.*= *-1.*$/ )
            {
               exit 1
            }
         }
      }
      if ( INBLOCK == 1 && $0 ~ /^[^:]+:/ )
      {
         exit 0
      }
      if ( INBLOCK == 0 && $0 ~ SEARCHPATTERN )
      {
         INBLOCK=1
      }
    }
' /etc/security/limits $1 2>/dev/null

return $?

}

function verify_prompt
{
   printOnScreen "Verifying command prompt requirements..."
   
   CURRENT_PROMPT=$(bash -l -i -c "echo \$PS1" | sed "s/^.*\(.\)[ \t]*$/\1/")
   
   if isDebug || ! ( [ "$CURRENT_PROMPT" == "#" ] || [ "$CURRENT_PROMPT" == "$" ] || [ "$CURRENT_PROMPT" == ">" ] )
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList prompt
   else
      printOnScreen "OK\n"
   fi
}

function verify_tmp
{
   printOnScreen "Verifying /tmp permissions..."
   
   if isDebug || ! ls -lad /tmp | grep "^drwxrwxrwt.*$" 1>/dev/null 2>/dev/null
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList tmp
   else
      printOnScreen "OK\n"
   fi  
}

function verify_aio
{
   printOnScreen "Verifying asynchronous I/O..."
   
   if isDebug || ! lsdev -F status -l aio0 | grep -i available 1>/dev/null 2>/dev/null
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList aio
   else
      printOnScreen "OK\n"
   fi
}

function verify_iso_en_US_module
{
   printOnScreen "Verifying locale module..."
   
   if isDebug || ! lslpp -l | grep bos.loc.iso.en_US 1>/dev/null 2>/dev/null
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList localModule
   else
      printOnScreen "OK\n"
   fi
}

function verify_home
{
   printOnScreen "Verifying /home for ctginst1..."
   
   if isDebug || [[ "`df -g -P /home | sed 's/  */;/g' | cut -d ';' -f 4 | tail -n 1 | sed 's/[.].*//'`" -lt "8" ]]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList homeDir
   else
      printOnScreen "OK\n"
   fi
}

if isCheckOperatingSystem || isCheckMiddleware
then
   verify_os
fi

if ! [ "$STATUS" == "$RC_UNKNOWNOS" ]
then
	# Call the verification steps if the operating system is ok
	if isCheckOperatingSystem
	then
	   verify_os
	   verify_disabledautomount
	   verify_pagingspace
	   verify_maxprocperuser
	   verify_largefilesizes
	   verify_prompt
	   verify_tmp
	fi
	
	if isCheckMiddleware
	then
	   verify_userlimits
	   if isAIX53
	   then
	      verify_aio
	   fi
	   verify_iso_en_US_module
	   verify_home
	fi
fi

printOnScreen "RC="

case $STATUS in
   $RC_OK)
      printOnScreen "OK\n";;
   $RC_WARNING)
      printOnScreen "WARNING\n";;
   $RC_ERROR)
      printOnScreen "ERROR\n";;
   $RC_UNKNOWNOS)
      printOnScreen "UNKNOWNOS\n";;
   *)
      printOnScreen ${STATUS};;
esac

printOutputList

exit $STATUS
