#!/bin/bash
##################################################################
# Licensed Materials - Property of IBM
# 5724-W78
# (C) Copyright IBM Corp. 2010
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
##################################################################

# Set variables to default values
RC_OK=0
RC_WARNING=10
RC_ERROR=20
RC_UNKNOWNOS=99

STATUS=$RC_OK

SILENT=0
SEPERATED=0
OUTPUTLIST=""
CHECK_OS=0
CHECK_MIDDLEWARE=0
DEBUG=0

TMPFILE=/tmp/tsam_lp.tmp

# If $DEBUG is 0, return false, if $DEBUG is 1, return true
function isDebug
{
        RETVAL=$(($(($DEBUG+1))%2))
        return $RETVAL
}

# Print text on screen if not called in silent mode
function printOnScreen
{
   if [ "$SILENT" == "0" ] && [ "$SEPERATED" == "0" ]
   then
      echo -en $1
   fi
	   
}

# Print usage on screen
function printUsage
{
   printOnScreen "Usage:\n"
   printOnScreen "   $0 [-c] [-s] -m\n"
   printOnScreen "   $0 [-c] [-s] -o\n"
}

while [ "$#" -ne "0" ]
do
   case $1 in
      "-s") SILENT=1; shift;;
      "-c") SEPERATED=1; shift;;
      "-m") CHECK_MIDDLEWARE=1; shift;;
      "-o") CHECK_OS=1; shift;;
      "-d") DEBUG=1; shift;;
      *) printUsage; exit 1;;
   esac
done

if [[ "$CHECK_MIDDLEWARE" == "0" ]] && [[ "$CHECK_OS" == "0" ]]
then
   printUsage;
fi

function isCheckMiddleware
{
   RETVAL=1;
   
   if [ "$CHECK_MIDDLEWARE" == "1" ]
   then
      RETVAL=0;
   fi
   
   return $RETVAL
}

function isCheckOperatingSystem
{
   RETVAL=1;
   
   if [ "$CHECK_OS" == "1" ]
   then
      RETVAL=0;
   fi
   
   return $RETVAL
}

function printOutputList
{
   if [ "$SILENT" == "0" ] && [ "$SEPERATED" == "1" ]
   then
      echo ${OUTPUTLIST} > ${TMPFILE}
   fi
}

function addToList
{
   if [ "$OUTPUTLIST" == "" ]
   then
      OUTPUTLIST=$1
   else
      OUTPUTLIST=${OUTPUTLIST},$1
   fi
}

function isSLESVersion
{
   VERSION_TO_CHECK=$1
   SP_TO_CHECK=$2
   
   SLES_VERSION_FILE=/etc/SuSE-release
   RET_VAL=1
   
   if test -f $SLES_VERSION_FILE && cat $SLES_VERSION_FILE | grep -e "^VERSION *= *$VERSION_TO_CHECK" 1>/dev/null 2>&1
   then
      if ! [ "$SP_TO_CHECK" == "" ]
      then
         if cat $SLES_VERSION_FILE | grep -e "^PATCHLEVEL *= *$SP_TO_CHECK" 1>/dev/null 2>&1
         then
            RET_VAL=0
         fi
      else
         RET_VAL=0
      fi
   fi
   
   return $RET_VAL
}

function isRHELVersion
{
   VERSION_TO_CHECK=$1
   SP_TO_CHECK=$2
   
   RHEL_VERSION_FILE=/etc/redhat-release
   RET_VAL=1
   
   GREP_REGEXP="release $VERSION_TO_CHECK"
   
   if ! [ "$SP_TO_CHECK" == "" ]
   then
      GREP_REGEXP=${GREP_REGEXP}.${SP_TO_CHECK}
   fi

   if test -f $RHEL_VERSION_FILE && cat $RHEL_VERSION_FILE | grep -e "$GREP_REGEXP" 1>/dev/null 2>&1
   then
      RET_VAL=0
   fi
   
   return $RET_VAL
}


# Verify that operating system is linux
function verify_os
{
   printOnScreen "Verifying operating System..."

   ERROR=false
   
   if isDebug || ! [[ "`uname -s`" =~ ^.*[lL]inux.*$ ]] || ! [[ "`getconf LONG_BIT`" == "64" ]]
   then
      ERROR=true
   else
      if ! isSLESVersion 10 3 && ! isSLESVersion 11 && ! isRHELVersion 5 4
      then
        ERROR=true
      fi
   fi
   
   if $ERROR
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_UNKNOWNOS}
      addToList os
   else
      printOnScreen "OK\n"
   fi
}

# Verify that the kernel version is 2.6
function verify_kernel
{
   printOnScreen "Verifiying kernel version..." 
   
      
   if isDebug || ! [[ "`uname -r`" =~ ^2[.]6.*$ ]]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList kernel
   else
      printOnScreen "OK\n"
   fi
}

# For RHEL or other operating systems, set SELinux to permissive or disabled
function verify_selinux
{
   printOnScreen "Verifying SELinux..."
   
   if isDebug || test -f /etc/redhat-release && ! [[ "`sestatus`" =~ ^.*disabled$ ]] && ! [[ "`sestatus`" =~ ^.*permissive$ ]]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList selinux
   else
      printOnScreen "OK\n"
   fi
}

# Disable automatic cleanup of the /tmp directory
function verify_tmpcleanup
{
   printOnScreen "Verifying automatic cleanup of the /tmp directory..."
   
   if isDebug || test -f /etc/redhat-release && test -f /etc/cron.daily/tmpwatch
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList tmp
   elif test -f /etc/SuSE-release && test -f /etc/cron.daily/suse.de-clean-tmp
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList tmp
   else
      printOnScreen "OK\n"
   fi 
}

# Set swap size
function verify_swap
{
   printOnScreen "Verifying the swap size..."
   
   SWAP=`free -o | sed "s/  */ /g" | grep Swap | cut -d ' ' -f 2`
   MEM=`free -o | sed "s/  */ /g" | grep Mem | cut -d ' ' -f 2`
   SWAP_MIN=$[MEM*2]
   
   if isDebug || ! [ "$SWAP" -ge "$SWAP_MIN" ]
   then
      if [ "$STATUS" == "$RC_OK" ]
      then
         STATUS=${RC_WARNING}
      fi
      printOnScreen "WARNING\n"
      addToList swap
   else
      printOnScreen "OK\n"
   fi
   
   
}

# Check a single kernel parameter
function verify_single_kernelparam
{
   PARAM=$1
   VALUE=$2
   
   FOUND=`sysctl -a | grep $PARAM | sed "s/[\t ]/ /g" | sed "s/^.* = \(.*\)$/\\1/"`
   
   printOnScreen " + $PARAM ..."
   
   EXPRESSION=false
   
   if [[ "$FOUND" =~ "^[0-9]*$" ]] && [[ "$VALUE" =~ "^[0-9]*$" ]]
   then
       # accept if the found value is in a range around the actual value to allow for small discrepancies
       VALUEMIN=$[$VALUE/100*98]
       VALUEMAX=$[$VALUE/100*102]

	   if [ "$FOUND" -gt "$VALUEMIN" ] && [ "$FOUND" -lt "$VALUEMAX" ]
	   then
	      EXPRESSION=true
	   fi
   else

       if [ "$FOUND" == "$VALUE" ]
       then
          EXPRESSION=true
       fi
   fi
   
   if isDebug || ! $EXPRESSION
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList "kernelparam!${PARAM}!${FOUND}!${VALUE}"
   else
      printOnScreen "OK\n"
   fi 
	   
}

function getNintyPercentFromMemoryInPages
{
   getMemoryInBytes
   PAGESIZE=$(getconf PAGESIZE)
   # calculate 90% of the total memory and return this in number of pages, i.e. divided by the page size
   CALCULATED=$[$CALCULATED/100*90/$PAGESIZE]
   export CALCULATED
}

function getMemoryInBytes
{
   MEMBYTES=`free -o -b | sed "s/  */ /g" | grep Mem | cut -d ' ' -f 2`
   # return the total memory in bytes
   CALCULATED=$MEMBYTES
   export CALCULATED
}

# Check kernel parameters for DB2
function verify_kernelparams
{
   printOnScreen "Verifying kernel parameters for db2...\n"
   
   getNintyPercentFromMemoryInPages
   shmallExpected=$CALCULATED
   getMemoryInBytes
   shmmaxExpected=$CALCULATED
   
   verify_single_kernelparam kernel.sem "250 256000 32 1024"
   verify_single_kernelparam kernel.shmmax $shmmaxExpected 
   verify_single_kernelparam kernel.shmall $shmallExpected
   verify_single_kernelparam kernel.msgmax 65536
   verify_single_kernelparam kernel.msgmnb 65536
}

function verify_single_userlimit
{
   PARAM=$1
   OPERATOR=$2
   VALUE=$3
   
   FOUND=`ulimit $PARAM`
   ERROR=false
   
   printOnScreen " + ulimit $PARAM ..."
   
   if [ "$OPERATOR" == "-ge" ]
   then
      if isDebug || ! [ "$FOUND" ==  "unlimited" ] && ! [ "$FOUND" -ge $VALUE ]
      then
         ERROR=true
      fi
   else      
      if isDebug || [ ! "$FOUND" $OPERATOR "$VALUE" ]
      then
         ERROR=true
      fi 
   fi
   
   if $ERROR
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList "userlimits!${PARAM}!${FOUND}!${VALUE}"
   else
      printOnScreen "OK\n"
   fi
}

# Check user limits
function verify_userlimits
{
   printOnScreen "Verifying user limits...\n"
   
   verify_single_userlimit -n -ge 8192
   verify_single_userlimit -s == unlimited
   
   if isSLESVersion 11
   then
      verify_single_userlimit -m == unlimited
      verify_single_userlimit -v == unlimited
   fi
}

# Verify command prompt requirements
function verify_prompt
{
   printOnScreen "Verifying command prompt requirements..."
   
   CURRENT_PROMPT=$(bash -l -i -c "echo \$PS1" | sed "s/^.*\(.\)[ \t]*$/\1/")
   
   if isDebug || ! ( [ "$CURRENT_PROMPT" == "#" ] || [ "$CURRENT_PROMPT" == "$" ] || [ "$CURRENT_PROMPT" == ">" ] )
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList prompt
   else
      printOnScreen "OK\n"
   fi
}

# Verify /tmp permissions to be 1777
function verify_tmpperm
{
   printOnScreen "Verifying /tmp permissions..."
   
   if isDebug || ! [[ "`ls -lad /tmp`" =~ ^drwxrwxrwt.*$ ]]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList tmp
   else
      printOnScreen "OK\n"
   fi    
}

function verify_libaio
{
   printOnScreen "Verifying asynchronous I/O..."
   
   if isDebug || ! test -f /usr/lib64/libaio.so.1
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList aio
   else
      printOnScreen "OK\n"
   fi
}

function verify_nsswitch
{
    printOnScreen "Verifying nsswitch.conf..."
    
    if isDebug || [ "`cat /etc/nsswitch.conf | grep -v "^#"| egrep "^((passwd)|(shadow)|(group))" | grep -cv ldap`" -lt "3" ]
    then
       printOnScreen "WARNING\n"
       if [ "$STATUS" == "$RC_OK" ]
       then
          STATUS=${RC_WARNING}
       fi
       addToList nsswitch
    else
       printOnScreen "OK\n"
    fi
}

function verify_home
{
   printOnScreen "Verifying /home for ctginst1..."
   
   if isDebug || [ "`df --sync -B 1G -P /home | sed "s/  */;/g" | cut -d ";" -f 4 | tail -n 1`" -lt "8" ]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList homeDir
   else
      printOnScreen "OK\n"
   fi
}

if isCheckOperatingSystem || isCheckMiddleware
then
   verify_os
fi

if ! [ "$STATUS" == "$RC_UNKNOWNOS" ]
then
	# Call the verification steps if the operating system is ok
	verify_userlimits
	
	if isCheckOperatingSystem
	then
	   verify_kernel
	   verify_selinux
	   verify_prompt
	   verify_tmpperm
	   verify_tmpcleanup
	   verify_swap
	fi
	
	if isCheckMiddleware
	then
	   verify_home
	   verify_kernelparams
	   verify_libaio
	   verify_nsswitch
	fi
fi

printOnScreen "RC="

case $STATUS in
   $RC_OK)
      printOnScreen "OK\n";;
   $RC_WARNING)
      printOnScreen "WARNING\n";;
   $RC_ERROR)
      printOnScreen "ERROR\n";;
   $RC_UNKNOWNOS)
      printOnScreen "UNKNOWNOS\n";;
   *)
      printOnScreen ${STATUS};;
esac

printOutputList

exit $STATUS
