#!/bin/bash
##################################################################
# Licensed Materials - Property of IBM
# 5724-W78
# (C) Copyright IBM Corp. 2010
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
##################################################################

# Set variables to default values
RC_OK=0
RC_WARNING=10
RC_ERROR=20
RC_UNKNOWNOS=99

STATUS=$RC_OK

SILENT=0
SEPERATED=0
MISSINGOUTPUTLIST=""
VALIDOUTPUTLIST=""
DEBUG=0

TMPFILE=/tmp/tsam_lp.tmp

# Print text on screen if not called in silent mode
function printOnScreen
{
   if [ "$SILENT" == "0" ] && [ "$SEPERATED" == "0" ]
   then
      echo -en $1
   fi
	   
}

# If $DEBUG is 0, return false, if $DEBUG is 1, return true
function isDebug
{
        RETVAL=$(($(($DEBUG+1))%2))
        return $RETVAL
}

# Print usage on screen
function printUsage
{
   printOnScreen "Usage:\n"
   printOnScreen "   $0 [-c] [-s]\n"
}

# process the input parms
while [ "$#" -ne "0" ]
do
   case $1 in
      "-s") SILENT=1; shift;;
      "-c") SEPERATED=1; shift;;
      "-d") DEBUG=1; shift;;
      *) printUsage; exit 1;;
   esac
done

function printMissingOutputList
{
   if [ "$SILENT" == "0" ] && [ "$SEPERATED" == "1" ]
   then
      echo ${MISSINGOUTPUTLIST} > ${TMPFILE}
   fi
}

function printValidOutputList
{
   if [ "$SILENT" == "0" ] && [ "$SEPERATED" == "1" ] && [ "$STATUS" == "$RC_OK" ]
   then
      echo ${VALIDOUTPUTLIST} > ${TMPFILE}
   fi
}

function addToList
{
   if [ "$MISSINGOUTPUTLIST" == "" ]
   then
      MISSINGOUTPUTLIST=$1
   else
      MISSINGOUTPUTLIST=${MISSINGOUTPUTLIST},$1
   fi
}

function addToValidList
{
   if [ "$VALIDOUTPUTLIST" == "" ]
   then
      VALIDOUTPUTLIST=$1
   else
      VALIDOUTPUTLIST=${VALIDOUTPUTLIST},$1
   fi
}

# Verify that operating system is linux
function verify_os
{
   printOnScreen "Verifying operating System..."

   if isDebug || ! [[ "`uname -s`" =~ ^.*[lL]inux.*$ ]]
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_UNKNOWNOS}
   else
      printOnScreen "OK\n"
   fi
}

function verify_package
{
   PACKAGENAME=$1
   ARCH=$2
   
   printOnScreen "Verifying package $PACKAGENAME"
   if ! [ "$ARCH" == "" ]
   then
      printOnScreen " ($ARCH)..."
   else
      printOnScreen "..."
   fi
   
   RPM_OUTPUT=`rpm -q -a --queryformat='%{N}-%{V}-%{R}.%{arch}\n' | grep $PACKAGENAME`
   RPM_RC=$?
   
   # if ARCH is specified, look for exact version  
   if [ "$RPM_RC" == "0" ] && ! [ "$ARCH" == "" ]
   then
      echo $RPM_OUTPUT | grep $ARCH 1> /dev/null 2> /dev/null
      RPM_RC=$?
   fi

   if ! [ "$RPM_RC" == "0" ] || isDebug
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList $PACKAGENAME
   else
      addToValidList $RPM_OUTPUT
      printOnScreen "OK\n"
   fi
}

function verify_lib
{
   LIBNAME=$1
   PATHNAME=$2
   
   printOnScreen "Verifying library ${PATHNAME}/${LIBNAME}..."
   
   if ! ls ${PATHNAME} | grep -e "^${LIBNAME}" 1>/dev/null 2>&1
   then
      printOnScreen "ERROR\n"
      STATUS=${RC_ERROR}
      addToList $LIBNAME
   else
      addToValidList $LIBNAME
      printOnScreen "OK\n"
   fi
}

function verify_rhel_packages
{
   verify_package compat-db
   verify_package compat-libstdc++ 64
   verify_package curl
   verify_package expect
   verify_package ftp
   verify_package libstdc++
   verify_package openssh
   verify_package perl
   verify_package procmail
   verify_package rpm-build
   verify_package tcl
   verify_package telnet
   verify_package tk
   verify_package wget
   verify_package xorg-x11-xfs
   verify_package xorg-x11-font-utils
   verify_lib libaio.so.1 /usr/lib64
   
   if [ "`cat /etc/redhat-release | sed "s/^.*release \([0-9]\).*$/\\1/"`" == "5" ]
   then
      verify_package compat-libstdc++ i386
      verify_package libXmu i386
      verify_package libXp- i386
      verify_package xorg-x11-fonts-base
      verify_package xorg-x11-fonts-75dpi
      verify_package xorg-x11-fonts-100dpi
      verify_lib libXtst.so /usr/lib64
      verify_lib libXft.so /usr/lib64
      verify_lib libXtst.so /usr/lib
      verify_lib libXft.so /usr/lib
      
      if [ "`uname -p`" == "s390x" ]
      then
      verify_package libaio
      fi
   fi
   
}

function verify_sles_packages
{
   verify_package curl
   verify_package expect
   verify_package gtk
   verify_package libstdc++
   verify_package libstdc++33-32bit
   verify_package ncftp
   verify_package openssh
   verify_package perl
   verify_package procmail
   verify_package tcl
   verify_package telnet
   verify_package tk
   verify_package wget
   verify_lib libaio.so.1 /usr/lib64
   
   if [ "`uname -p`" == "s390x" ]
   then
      verify_package libaio
      verify_lib libstdc++.so.5 /usr/lib64
   fi
}


# verify base os
verify_os

# verify the packages (depends on distribution)
if test -f /etc/redhat-release
then
   verify_rhel_packages
elif test -f /etc/SuSE-release
then
   verify_sles_packages
else
   STATUS=${RC_UNKNOWNOS}
fi

# process the output
printOnScreen "RC="

case $STATUS in
   $RC_OK)
      printOnScreen "OK\n";;
   $RC_ERROR)
      printOnScreen "ERROR\n";;
   $RC_UNKNOWNOS)
      printOnScreen "UNKNOWNOS\n";;
   *)
      printOnScreen ${STATUS};;
esac

# write to the interface file for the launchpad
printMissingOutputList
printValidOutputList

exit $STATUS
